
# iterative correction and eigenvector decomposition - cis + trans
# intput: raw matrix, maximum iterations, error tolerance
# output: normalized matrix

ice <- function(mat, max_iters = 100, th = 1e-3) {

  nr <- nrow(mat)
  nc <- ncol(mat)

  pre_bias_i <- integer(nr)
  pre_bias_j <- integer(nc)
  real_iters <- 0

  for (m in 1:max_iters) {
	  # step 1: calculate the sum of the matrix over all rows
	  rsum <- rowSums(mat)
	  csum <- colSums(mat)
	  # step 2:
	  # step 3: calculate vector of biasa
	  bias_i <- rsum
	  bias_j <- csum
	  # step 4: renormalize bias vector by its mean value over non-zero bins to avoid numerical instabilities
	  bias_meani <- mean(bias_i[bias_i != 0])
	  bias_i <- bias_i / bias_meani
	  bias_meanj <- mean(bias_j[bias_j != 0])
	  bias_j <- bias_j / bias_meanj
	  # step 5: set zero values of bias to one to avoid 0/0 error
	  bias_i[bias_i == 0] <- 1
	  bias_j[bias_j == 0] <- 1
	  # step 6: divide mat_ij by bias_i*bias_j for all ij
	  expectedMat <- matrix(bias_i, nrow = nr) %*% bias_j
	  mat <- mat / expectedMat
	  # step 7:
    if (m > 1) {
		  tmp_sum_i <- sum(abs(pre_bias_i - bias_i))
		  tmp_sum_j <- sum(abs(pre_bias_j - bias_j))
		  if (tmp_sum_i < th && tmp_sum_j < th) break
	  }  	
	  pre_bias_i <- bias_i
	  pre_bias_j <- bias_j
	  real_iters <- real_iters + 1
  }
	mat <- round(mat, 6)

  return(mat)
}


