
# Knight-Ruiz Knight-Ruiz 2-norm (KR2) - cis + trans
# input: mat_ori
# output: a list

kr2 <- function(mat_ori, tol = 1e-6) {

  #mat_pre <- mat_ori^2
	mat_pre <- mat_ori

  index_sym <- TRUE
	# check if the matrix is symmetric
  if (nrow(mat_pre) != ncol(mat_pre)) {
	  index_sym <- FALSE
  } else {
	  rownames(mat_pre) <- colnames(mat_pre)
	  if (! isSymmetric(mat_pre)) index_sym <- FALSE
  }
	# remove rows and columns with all zero values
  index_rows <- which(rowSums(mat_pre) != 0)
  index_cols <- which(colSums(mat_pre) != 0)
  mat <- NULL
  num_rows <- length(index_rows)
  num_cols <- length(index_cols)

  if (index_sym) { # symmetric matrix
	  mat <- mat_pre[index_rows, index_cols]
  } else { # non-symmetric matrix
    tmp_mat <- mat_pre[index_rows, index_cols]
	  mat <- rbind(cbind(matrix(0, num_rows, num_rows), tmp_mat), 
		             cbind(t(tmp_mat), matrix(0, num_cols, num_cols)))
  }

  n <- nrow(mat)
  e <- rep(1, n)
  x0 <- e
  delta <- 0.1
  Delta <- 3
  fl <- 0
  # Inner stopping criterion parameters
  g <- 0.9
  etamax <- 0.1
  eta <- etamax
  stop_tol <- tol * 0.5
  x <- x0
  rt <- tol ^ 2
  v <- x * (mat %*% x)
  rk <- 1 - v
  rho_km1 <- as.vector(t(rk) %*% rk)
  rout <- rho_km1
  rold <- rout
  MVP <- 0
  i <- 0 # Outer iteration count

  while (rout > rt) { # Outer iteration
    i <- i + 1
    k <- 0
    y <- e
    innertol <- max(eta ^ 2 * rout, rt)
    while (rho_km1 > innertol) { # Inner iteration by CG
      k <- k + 1
      if (k == 1) {
        Z <- rk / v
        p <- Z
        rho_km1 <- t(rk) %*% Z
      } else {
        beta <- as.vector(rho_km1 / rho_km2)
        p <- Z + beta * p
      }
      # Update search direction efficiently
      w <- x * (mat %*% (x * p)) + v * p
      alpha <- as.vector(rho_km1 / (t(p) %*% w))
      ap <- alpha * p
      # Test distance to boundary of cone
      ynew <- y + ap
      if (min(ynew) <= delta) {
        if (delta == 0) { break }
        ind <- which(ap < 0)
        gamma <- min((delta - y[ind]) / ap[ind])
        y <- y + gamma * ap
        break
      }
      if (max(ynew) >= Delta) {
        ind <- which(ynew > Delta)
        gamma <- min((Delta - y[ind]) / ap[ind])
        y <- y + gamma * ap
        break
      }
      y <- ynew
      rk <- rk - alpha * w
      rho_km2 <- rho_km1
      Z <- rk / v
      rho_km1 <- t(rk) %*% Z
    } # End inner iteration
    x <- x * y
    v <- x * (mat %*% x)
    rk <- 1 - v
    rho_km1 <- as.vector(t(rk) %*% rk)
    rout <- rho_km1
    MVP <- MVP + k + 1
    # Update inner iteration stopping criterion
    rat <- as.vector(rout / rold)
    rold <- rout
    res_norm <- as.vector(sqrt(rout))
    eta_o <- eta
    eta <- g * rat
    if (g * eta_o ^ 2 > 0.1) {
      eta <- max(eta, g * eta_o ^ 2)
    }
    eta <- max(min(eta, etamax), stop_tol / res_norm)
    if (fl == 1) {
      res <- c(res, r_norm)
    }
  }
  mat_x <- diag(as.vector(x), nrow = length(x))
	if (index_sym) { # symmetric matrix
	  mat <- mat_x %*% mat %*% mat_x
		tmp_mat <- mat_ori[index_rows, index_cols]
		mat[tmp_mat < 0] <- -1 * mat[tmp_mat < 0]
    mat <- round(mat, 6)
  } else { # non-symmetric matrix
		tmp_mat <- mat_ori[index_rows, index_cols]
    mat <- (mat_x %*% mat %*% mat_x)[1:num_rows, (num_rows + 1):(num_rows + num_cols)]
		mat[tmp_mat < 0] <- -1 * mat[tmp_mat < 0]
    mat <- round(, 6)
  }
	
	return(list(mat = mat, indexRows = index_rows, indexColumns = index_cols))
}


