
# input: cosegregation file and target chromosome
# output: a list contains "ld" for raw contact matrix and 
#													"nld" for normalized linkage disequilibrium
#                         "wdf" window detection frequency bias
#                         "coc" cutting orientation and compactness bias

cal_coc <- function(x) {
  tmp_biases <- rep(0, length(x))
  if (all(x ==0)) return(tmp_biases)
  index_nonzero <- which(x != 0)
  index_segment_start <- unique(c(index_nonzero[1], index_nonzero[diff(c(0, index_nonzero)) > 1]))
  index_segment_end <- index_nonzero[c(which(diff(c(0, index_nonzero)) > 1) - 1, length(index_nonzero))]
  for (i in 1:length(index_segment_start)) {
    p1 <- index_segment_start[i]
    p2 <- index_segment_end[i]
    tmp_biases[p1:p2] <- p2 - p1 + 1
  }
  return(tmp_biases)
}

ld_nld_cis <- function(file_coseg, chr = "chr1") {

	dat_coseg <- read.table(file_coseg, header = TRUE)
	dat_chr <- subset(dat_coseg, chrom == chr)
  # remove "chrom start stop" columns
  mat_chr <- as.matrix(dat_chr[ , c(-1, -2, -3)]) 
	# number of bins for this chromosome
	nr <- nrow(mat_chr) 
	# number of nuclear profiles
	nc <- ncol(mat_chr) 
	
	mat_ld <- matrix(0, nr, nr)
	mat_nld <- matrix(0, nr, nr)

  rsum <- rowSums(mat_chr)
  for (i in 1:nr) {
    for (j in i:nr) {
      fa <- rsum[i] / nc
      fb <- rsum[j] / nc
      dij <- sum(mat_chr[i, ] & mat_chr[j, ]) / nc - fa * fb
      mat_ld[i, j] <- dij
      mat_ld[j, i] <- dij
			dmax <- 0
			ifelse(dij <= 0, dmax <- min(fa * fb, (1 - fa) * (1 - fb)), dmax <- min(fb * (1 - fa), fa * (1-fb)))
		  ifelse(dmax, mat_nld[i, j] <- dij / dmax, mat_nld[i, j] <- dij)
			mat_nld[j, i] <- mat_nld[i, j]
    }
  }
	mat_ld <- round(mat_ld, 6)
	mat_nld <- round(mat_nld, 6)
	bias1 <- round(rsum / nc, 6)
	bias2 <- rowSums(apply(mat_chr, 2, cal_coc))
  bias2 <- round(bias2 / nc, 6)

	return(list(ld = mat_ld, nld = mat_nld, wdf = bias1, coc = bias2))
}


