
# input: cosegregation file and two target chromosomes
# output: a list contains "ld" for raw contact matrix and 
#													"nld" for normalized linkage disequilibrium
#                         "wdf1 and wdf2"  window detection frequency bias for the two chrs
#                         "coc1 and coc2" cutting orientation and compactness bias for the two chrs

cal_coc <- function(x) {
  tmp_biases <- rep(0, length(x))
  if (all(x ==0)) return(tmp_biases)
  index_nonzero <- which(x != 0)
  index_segment_start <- unique(c(index_nonzero[1], index_nonzero[diff(c(0, index_nonzero)) > 1]))
  index_segment_end <- index_nonzero[c(which(diff(c(0, index_nonzero)) > 1) - 1, length(index_nonzero))]
  for (i in 1:length(index_segment_start)) {
    p1 <- index_segment_start[i]
    p2 <- index_segment_end[i]
    tmp_biases[p1:p2] <- p2 - p1 + 1
  }
  return(tmp_biases)
}

ld_nld_trans <- function(file_coseg, chr1 = "chr1", chr2 = "chr2") {

	dat_coseg <- read.table(file_coseg, header = TRUE)

	dat_chr1 <- subset(dat_coseg, chrom == chr1)
	dat_chr2 <- subset(dat_coseg, chrom == chr2)
  # remove "chrom start stop" columns
  mat_chr1 <- as.matrix(dat_chr1[ , c(-1, -2, -3)]) 
  mat_chr2 <- as.matrix(dat_chr2[ , c(-1, -2, -3)]) 
	# number of bins for the two chrs
	nr1 <- nrow(mat_chr1) 
	nr2 <- nrow(mat_chr2) 
	# number of nuclear profiles
	nc <- ncol(mat_chr1) 
	
	mat_ld <- matrix(0, nr1, nr2)
	mat_nld <- matrix(0, nr1, nr2)

  rsum1 <- rowSums(mat_chr1)
  rsum2 <- rowSums(mat_chr2)

  for (i in 1:nr1) {
    for (j in 1:nr2) {
      fa <- rsum1[i] / nc
      fb <- rsum2[j] / nc
      dij <- sum(mat_chr1[i, ] & mat_chr2[j, ]) / nc - fa * fb
      mat_ld[i, j] <- dij
			dmax <- 0
			ifelse(dij <= 0, dmax <- min(fa * fb, (1 - fa) * (1 - fb)), dmax <- min(fb * (1 - fa), fa * (1-fb)))
		  ifelse(dmax, mat_nld[i, j] <- dij / dmax, mat_nld[i, j] <- dij)
    }
  }
	mat_ld <- round(mat_ld, 6)
	mat_nld <- round(mat_nld, 6)
	
	bias1_1 <- round(rsum1 / nc, 6)
	bias1_2 <- round(rsum2 / nc, 6)

	bias2_1 <- rowSums(apply(mat_chr1, 2, cal_coc))
  bias2_1 <- round(bias2_1 / nc, 6)
	bias2_2 <- rowSums(apply(mat_chr2, 2, cal_coc))
	bias2_2 <- round(bias2_2 / nc, 6)

	return(list(ld = mat_ld, nld = mat_nld, wdf1 = bias1_1, wdf2 = bias1_2, coc1 = bias2_1, coc2 = bias2_2))
}


